@@ -12,19 +12,25 @@ import numpy
 import os
 import scipy
 import typing
+import warnings
+from typing import Union
 
 from . import utils
+from . utils import pm
 
 
-__version__ = '6.1.2'
+__version__ = '6.5.2'
 _log = logging.getLogger('calibr8')
 
 
-class NumericPosterior:
-    """ The result of a numeric infer_independent operation. """
+class InferenceResult:
+    """Generic base type of independent value inference results."""
+
+
+class ContinuousInference(InferenceResult):
+    """Describes properties common for inference with continuous independent variables."""
     def __init__(
         self,
-        median: float,
         eti_x: numpy.ndarray,
         eti_pdf: numpy.ndarray,
         eti_prob: float,
@@ -36,8 +42,6 @@ class NumericPosterior:
 
         Parameters
         ----------
-        median : float
-            x-value of the posterior median
         eti_x : array
             Values of the independent variable in [eti_lower, eti_upper]
         eti_pdf : array
@@ -51,52 +55,105 @@ class NumericPosterior:
         hdi_prob : float
             Probability mass in the HDI
         """
-        self.median = median
-        self.eti_x = eti_x
-        self.eti_pdf = eti_pdf
-        self.eti_prob = eti_prob
-        self.hdi_x = hdi_x
-        self.hdi_pdf = hdi_pdf
-        self.hdi_prob = hdi_prob
-
-    def __repr__(self) -> str:
-        result = (
-            str(type(self))
-            + f"\n    ETI ({round(self.eti_prob, 3) * 100:.1f} %): [{round(self.eti_lower, 4)}, {round(self.eti_upper, 4)}] Δ={round(self.eti_width, 4)}"
-            + f"\n    HDI ({round(self.hdi_prob, 3) * 100:.1f} %): [{round(self.hdi_lower, 4)}, {round(self.hdi_upper, 4)}] Δ={round(self.hdi_width, 4)}"
-        )
-        return result
+        self._eti_x = eti_x
+        self._eti_pdf = eti_pdf
+        self._eti_prob = eti_prob
+        self._hdi_x = hdi_x
+        self._hdi_pdf = hdi_pdf
+        self._hdi_prob = hdi_prob
 
     @property
-    def eti_lower(self) -> float:
+    def eti_x(self) -> numpy.ndarray:
+        """Values of the independent variable in the interval [eti_lower, eti_upper]"""
+        return self._eti_x
+
+    @property
+    def eti_pdf(self) -> numpy.ndarray:
+        """Values of the posterior probability density at the positions `eti_x`."""
+        return self._eti_pdf
+
+    @property
+    def eti_lower(self) -> Union[float, numpy.ndarray]:
         """ Lower bound of the ETI. This is the first value in `eti_x`. """
-        return self.eti_x[0]
+        return self._eti_x[..., 0]
 
     @property
-    def eti_upper(self) -> float:
+    def eti_upper(self) -> Union[float, numpy.ndarray]:
         """ Upper bound of the ETI. This is the last value in `eti_x`. """
-        return self.eti_x[-1]
+        return self._eti_x[..., -1]
 
     @property
-    def eti_width(self) -> float:
+    def eti_width(self) -> Union[float, numpy.ndarray]:
         """ Width of the ETI. """
         return self.eti_upper - self.eti_lower
 
     @property
-    def hdi_lower(self) -> float:
+    def eti_prob(self) -> float:
+        """ Probability mass of the given equal tailed interval. """
+        return self._eti_prob
+
+    @property
+    def hdi_x(self) -> numpy.ndarray:
+        """Values of the independent variable in the interval [hdi_lower, hdi_upper]"""
+        return self._hdi_x
+
+    @property
+    def hdi_pdf(self) -> numpy.ndarray:
+        """Values of the posterior probability density at the positions `hdi_x`."""
+        return self._hdi_pdf
+
+    @property
+    def hdi_lower(self) -> Union[float, numpy.ndarray]:
         """ Lower bound of the HDI. This is the first value in `hdi_x`. """
-        return self.hdi_x[0]
+        return self._hdi_x[..., 0]
 
     @property
-    def hdi_upper(self) -> float:
+    def hdi_upper(self) -> Union[float, numpy.ndarray]:
         """ Upper bound of the HDI. This is the last value in `hdi_x`. """
-        return self.hdi_x[-1]
+        return self._hdi_x[..., -1]
 
     @property
-    def hdi_width(self) -> float:
+    def hdi_width(self) -> Union[float, numpy.ndarray]:
         """ Width of the HDI. """
         return self.hdi_upper - self.hdi_lower
 
+    @property
+    def hdi_prob(self) -> float:
+        """ Probability mass of the given highest density interval. """
+        return self._hdi_prob
+
+
+class ContinuousUnivariateInference(ContinuousInference):
+    """ The result of a numeric infer_independent operation with a univariate model. """
+    def __init__(
+        self,
+        median: float,
+        **kwargs,
+    ) -> None:
+        """ The result of a numeric infer_independent operation.
+
+        Parameters
+        ----------
+        median : float
+            x-value of the posterior median
+        **kwargs : dict
+            Forwarded to InferenceResult base constructor.
+        """
+        self.median = median
+        super().__init__(**kwargs)
+
+    def __repr__(self) -> str:
+        result = (
+            str(type(self))
+            + f"\n    ETI ({numpy.round(self.eti_prob, 3) * 100:.1f} %): [{numpy.round(self.eti_lower, 4)}, {numpy.round(self.eti_upper, 4)}] Δ={round(self.eti_width, 4)}"
+            + f"\n    HDI ({numpy.round(self.hdi_prob, 3) * 100:.1f} %): [{numpy.round(self.hdi_lower, 4)}, {numpy.round(self.hdi_upper, 4)}] Δ={round(self.hdi_width, 4)}"
+        )
+        return result
+
+
+class ContinuousMultivariateInference(ContinuousInference):
+    """The result of a multivariate independent variable inference."""
+
 
 def _interval_prob(x_cdf: numpy.ndarray, cdf: numpy.ndarray, a: float, b: float):
     """Calculates the probability in the interval [a, b]."""
@@ -143,7 +200,7 @@ def _get_hdi(
     guess_upper: float,
     *,
     history: typing.Optional[typing.DefaultDict[str, typing.List]]=None
-) -> typing.Tuple[float]:
+) -> typing.Tuple[float, float]:
     """ Find the highest density interval (HDI) corresponding to a certain credible interval probability level.
 
     Parameters
@@ -198,10 +255,15 @@ def _get_hdi(
             history["L"].append(L)
         return L
 
+    initial_guess = [guess_lower, guess_upper - guess_lower]
+    if not numpy.isfinite(hdi_objective(initial_guess)):
+        # Bad initial guess. Reset to the outer limimts.
+        initial_guess = [x_cdf[0], x_cdf[-1] - x_cdf[0]]
+
     fit = scipy.optimize.fmin(
         hdi_objective,
         # parametrize as b=a+d
-        x0=[guess_lower, guess_upper - guess_lower],
+        x0=initial_guess,
         xtol=numpy.ptp(x_cdf) / len(x_cdf),
         disp=False
     )
@@ -209,10 +271,45 @@ def _get_hdi(
     hdi_upper = hdi_lower + hdi_width
     return hdi_lower, hdi_upper
 
-class CalibrationModel:
+
+class DistributionMixin:
+    """Maps the values returned by `CalibrationModel.predict_dependent`
+    to a SciPy distribution and its parameters, and optionally also to
+    a PyMC distribution and its parameters.
+    """
+    scipy_dist = None
+    pymc_dist = None
+
+    def to_scipy(*args) -> dict:
+        raise NotImplementedError("This model does not implement a mapping to SciPy distribution parameters.")
+
+    def to_pymc(*args) -> dict:
+        raise NotImplementedError("This model does not implement a mapping to PyMC distribution parameters.")
+
+
+def _inherits_noisemodel(cls):
+    """Determines if cls is a sub-type of DistributionMixin that's not DistributionMixin or a calibration model."""
+    for m in cls.__mro__:
+        if (
+            issubclass(m, DistributionMixin)
+            and m is not DistributionMixin
+            and not issubclass(m, CalibrationModel)
+        ):
+            return True
+    return False
+
+
+class CalibrationModel(DistributionMixin):
     """A parent class providing the general structure of a calibration model."""
-    
-    def __init__(self, independent_key:str, dependent_key:str, *, theta_names:typing.Tuple[str]):
+
+    def __init__(
+        self,
+        independent_key:str,
+        dependent_key:str,
+        *,
+        theta_names:typing.Tuple[str],
+        ndim: int,
+    ):
         """Creates a CalibrationModel object.
 
         Parameters
@@ -223,7 +320,17 @@ class CalibrationModel:
             name of the dependent variable
         theta_names : optional, tuple of str
             names of the model parameters
+        ndim : int
+            Number of independent dimensions in the model.
         """
+        if not _inherits_noisemodel(type(self)):
+            warnings.warn(
+                "This model does not implement a noise model yet."
+                "\nAdd a noise model mixin to your class definition. For example:"
+                "\n`class MyModel(CalibrationModel, LaplaceNoise)`",
+                DeprecationWarning,
+                stacklevel=2
+            )
         # make sure that the inheriting type has no required constructor (kw)args
         args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations = inspect.getfullargspec(type(self).__init__)
         n_defaults = 0 if not defaults else len(defaults)
@@ -237,6 +344,7 @@ class CalibrationModel:
         self.__theta_fitted = None
 
         # public attributes/properties
+        self.ndim = ndim
         self.independent_key = independent_key
         self.dependent_key = dependent_key
         self.theta_names = theta_names
@@ -256,6 +364,11 @@ class CalibrationModel:
     @theta_fitted.setter
     def theta_fitted(self, value: typing.Optional[typing.Sequence[float]]):
         if value is not None:
+            if numpy.shape(value) != numpy.shape(self.theta_names):
+                raise ValueError(
+                    f"The number of parameters ({len(value)}) "
+                    f"does not match the number of parameter names ({len(self.theta_names)})."
+                )
             self.__theta_fitted = tuple(value)
             self.__theta_timestamp = datetime.datetime.now().astimezone(datetime.timezone.utc).replace(microsecond=0)
         else:
@@ -306,14 +419,14 @@ class CalibrationModel:
         raise NotImplementedError('The predict_independent function should be implemented by the inheriting class.')
 
     def infer_independent(
-        self, y:typing.Union[int,float,numpy.ndarray], *, 
-        lower:float, upper:float, steps:int=300, 
-        ci_prob:float=1
-    ) -> NumericPosterior:
+        self,
+        y:Union[int,float,numpy.ndarray],
+        *,
+        lower,
+        upper,
+    ) -> InferenceResult:
         """Infer the posterior distribution of the independent variable given the observations of the dependent variable.
-        The calculation is done numerically by integrating the likelihood in a certain interval [upper,lower]. 
-        This is identical to the posterior with a Uniform (lower,upper) prior. If precentiles are provided, the interval of
-        the PDF will be shortened.
+        A Uniform (lower,upper) prior is applied.
 
         Parameters
         ----------
@@ -323,76 +436,25 @@ class CalibrationModel:
             lower limit for uniform distribution of prior
         upper : float
             upper limit for uniform distribution of prior
-        steps : int
-            steps between lower and upper or steps between the percentiles (default 300)
-        ci_prob : float
-            Probability level for ETI and HDI credible intervals.
-            If 1 (default), the complete interval [upper,lower] will be returned, 
-            else the PDFs will be trimmed to the according probability interval; 
-            float must be in the interval (0,1]
 
         Returns
         -------
-        posterior : NumericPosterior
-            the result of the numeric posterior calculation
-        """  
-        y = numpy.atleast_1d(y)
+        posterior : InferenceResult
+            Result of the independent variable inference.
+        """
+        raise NotImplementedError(f"This calibration model does not implement an .infer_independent() method.")
 
-        likelihood_integral, _ = scipy.integrate.quad(
-            func=lambda x: self.likelihood(x=x, y=y),
-            # by restricting the integral into the interval [a,b], the resulting PDF is
-            # identical to the posterior with a Uniform(a, b) prior.
-            # 1. prior probability is constant in [a,b]
-            # 2. prior probability is 0 outside of [a,b]
-            # > numerical integral is only computed in [a,b], but because of 1. and 2., it's
-            #   identical to the integral over [-∞,+∞]
-            a=lower, b=upper,
-        )
-
-        # high resolution x-coordinates for integration
-        # the first integration is just to find the peak
-        x_integrate = numpy.linspace(lower, upper, 10_000)
-        area = scipy.integrate.cumtrapz(self.likelihood(x=x_integrate, y=y, scan_x=True), x_integrate, initial=0)
-        cdf = area / area[-1]
-
-        # now we find a high-resolution CDF for (1-shrink) of the probability mass
-        shrink = 0.00001
-        xfrom, xto = _get_eti(x_integrate, cdf, 1 - shrink)
-        x_integrate = numpy.linspace(xfrom, xto, 100_000)
-        area = scipy.integrate.cumtrapz(self.likelihood(x=x_integrate, y=y, scan_x=True), x_integrate, initial=0)
-        cdf = (area / area[-1]) * (1 - shrink) + shrink / 2
-
-        # TODO: create a smart x-vector from the CDF with varying stepsize
-
-        if ci_prob != 1:
-            if not (0 < ci_prob <= 1):
-                raise ValueError(f'Unexpected `ci_prob` value of {ci_prob}. Expected float in interval (0, 1].')
-
-            # determine the interval bounds from the high-resolution CDF
-            eti_lower, eti_upper = _get_eti(x_integrate, cdf, ci_prob)
-            hdi_lower, hdi_upper = _get_hdi(x_integrate, cdf, ci_prob, eti_lower, eti_upper, history=None)
-
-            eti_x = numpy.linspace(eti_lower, eti_upper, steps)
-            hdi_x = numpy.linspace(hdi_lower, hdi_upper, steps)
-            eti_pdf = self.likelihood(x=eti_x, y=y, scan_x=True) / likelihood_integral
-            hdi_pdf = self.likelihood(x=hdi_x, y=y, scan_x=True) / likelihood_integral
-            eti_prob = _interval_prob(x_integrate, cdf, eti_lower, eti_upper)
-            hdi_prob = _interval_prob(x_integrate, cdf, hdi_lower, hdi_upper)
-        else:
-            x = numpy.linspace(lower, upper, steps)
-            eti_x = hdi_x = x
-            eti_pdf = hdi_pdf = self.likelihood(x=x, y=y, scan_x=True) / likelihood_integral
-            eti_prob = hdi_prob = 1
-
-        median = x_integrate[numpy.argmin(numpy.abs(cdf - 0.5))]
-
-        return NumericPosterior(
-            median,
-            eti_x, eti_pdf, eti_prob,
-            hdi_x, hdi_pdf, hdi_prob,
-        )
-
-    def loglikelihood(self, *, y, x, theta=None):
+    def loglikelihood(
+        self,
+        *,
+        y,
+        x,
+        name: str=None,
+        replicate_id: str=None,
+        dependent_key: str=None,
+        theta=None,
+        **dist_kwargs
+    ):
         """ Loglikelihood of observation (dependent variable) given the independent variable.
 
         If both x and y are 1D-vectors, they must have the same length and the likelihood will be evaluated elementwise.
@@ -406,15 +468,79 @@ class CalibrationModel:
             observed measurements (dependent variable)
         x : scalar, array-like or TensorVariable
             assumed independent variable
+        name : str
+            Name for the likelihood variable in a PyMC model (tensor mode).
+            Previously this was `f'{replicate_id}.{dependent_key}'`.
+        replicate_id : optional, str
+            Deprecated; pass the `name` kwarg instead.
+        dependent_key : optional, str
+            Deprecated; pass the `name` kwarg instead.
         theta : optional, array-like
-            model parameters
+            Parameters for the calibration model to use instead of `theta_fitted`.
+            The vector must have the correct length, but can have numeric and or symbolic entries.
+            Use this kwarg to run MCMC on calibration model parameters.
+        **dist_kwargs : dict
+            Additional keyword arguments are forwarded to the PyMC distribution.
+            Most prominent example: `dims`.
 
         Returns
         -------
         L : float or TensorVariable
             sum of log-likelihoods
         """
-        raise NotImplementedError('The loglikelihood function should be implemented by the inheriting class.')
+        if theta is None:
+            if self.theta_fitted is None:
+                raise Exception('No parameter vector was provided and the model is not fitted with data yet.')
+            theta = self.theta_fitted
+
+        if not isinstance(x, (list, numpy.ndarray, float, int)) and not utils.istensor(x):
+            raise ValueError(
+                f'Input x must be a scalar, TensorVariable or an array-like object, but not {type(x)}'
+            )
+        if not isinstance(y, (list, numpy.ndarray, float, int)) and not utils.istensor(x):
+            raise ValueError(
+                f'Input y must be a scalar or an array-like object, but not {type(y)}'
+            )
+
+        params = self.predict_dependent(x, theta=theta)
+        if utils.istensor(x) or utils.istensor(theta):
+            if pm.Model.get_context(error_if_none=False) is not None:
+                if replicate_id and dependent_key:
+                    warnings.warn(
+                        "The `replicate_id` and `dependent_key` parameters are deprecated. Use `name` instead.",
+                        DeprecationWarning
+                    )
+                    name = f'{replicate_id}.{dependent_key}'
+                if not name:
+                    raise ValueError("A `name` must be specified for the PyMC likelihood.")
+                rv = self.pymc_dist(
+                    name,
+                    **self.to_pymc(*params),
+                    observed=y,
+                    **dist_kwargs or {}
+                )
+            else:
+                rv = self.pymc_dist.dist(
+                    **self.to_pymc(*params),
+                    **dist_kwargs or {}
+                )
+            # The API to get log-likelihood tensors differs between PyMC versions
+            if pm.__version__[0] == "3":
+                if isinstance(rv, pm.model.ObservedRV):
+                    return rv.logpt.sum()
+                elif isinstance(rv, pm.Distribution):
+                    return rv.logp(y).sum()
+            else:
+                return pm.joint_logpt(rv, y, sum=True)
+        else:
+            logp = None
+            if hasattr(self.scipy_dist, "logpdf"):
+                logp = self.scipy_dist.logpdf
+            elif hasattr(self.scipy_dist, "logpmf"):
+                logp = self.scipy_dist.logpmf
+            else:
+                raise NotImplementedError("No logpdf or logpmf methods found on {self.scipy_dist}.")
+            return logp(y, **self.to_scipy(*params)).sum(axis=-1)
 
     def likelihood(self, *, y, x, theta=None, scan_x: bool=False):
         """ Likelihood of observation (dependent variable) given the independent variable.
@@ -441,8 +567,8 @@ class CalibrationModel:
             try:
                 # Try to pass `x` as a column vector to benefit from broadcasting
                 # if that's implemented by the underlying model.
-                result = numpy.exp(self.loglikelihood(y=y, x=x[:, None], theta=theta))
-                if not result.shape == x.shape:
+                result = numpy.exp(self.loglikelihood(y=y, x=x[..., None], theta=theta))
+                if not numpy.shape(result) == numpy.shape(x):
                     raise ValueError("The underlying model does not seem to implement broadcasting.")
                 return result
             except:
@@ -555,6 +681,109 @@ class CalibrationModel:
         return obj
 
 
+class ContinuousUnivariateModel(CalibrationModel):
+    def __init__(self, independent_key: str, dependent_key: str, *, theta_names: typing.Tuple[str]):
+        super().__init__(independent_key, dependent_key, theta_names=theta_names, ndim=1)
+
+    def infer_independent(
+        self,
+        y:Union[int, float, numpy.ndarray],
+        *,
+        lower:float,
+        upper:float,
+        steps:int=300,
+        ci_prob:float=1,
+    ) -> ContinuousUnivariateInference:
+        """Infer the posterior distribution of the independent variable given the observations of the dependent variable.
+        The calculation is done numerically by integrating the likelihood in a certain interval [upper,lower]. 
+        This is identical to the posterior with a Uniform (lower,upper) prior.
+
+        Parameters
+        ----------
+        y : int, float, array
+            One or more observations at the same x.
+        lower : float
+            Lower limit for uniform distribution of prior.
+        upper : float
+            Upper limit for uniform distribution of prior.
+        steps : int
+            Steps between lower and upper or steps between the percentiles (default 300).
+        ci_prob : float
+            Probability level for ETI and HDI credible intervals.
+            If 1 (default), the complete interval [upper,lower] will be returned,
+            else the PDFs will be trimmed to the according probability interval;
+            float must be in the interval (0,1]
+
+        Returns
+        -------
+        posterior : ContinuousUnivariateInference
+            Result of the numeric posterior calculation.
+        """  
+        y = numpy.atleast_1d(y)
+
+        likelihood_integral, _ = scipy.integrate.quad(
+            func=lambda x: self.likelihood(x=x, y=y),
+            # by restricting the integral into the interval [a,b], the resulting PDF is
+            # identical to the posterior with a Uniform(a, b) prior.
+            # 1. prior probability is constant in [a,b]
+            # 2. prior probability is 0 outside of [a,b]
+            # > numerical integral is only computed in [a,b], but because of 1. and 2., it's
+            #   identical to the integral over [-∞,+∞]
+            a=lower, b=upper,
+        )
+
+        # high resolution x-coordinates for integration
+        # the first integration is just to find the peak
+        x_integrate = numpy.linspace(lower, upper, 10_000)
+        area = scipy.integrate.cumtrapz(self.likelihood(x=x_integrate, y=y, scan_x=True), x_integrate, initial=0)
+        cdf = area / area[-1]
+
+        # now we find a high-resolution CDF for (1-shrink) of the probability mass
+        shrink = 0.00001
+        xfrom, xto = _get_eti(x_integrate, cdf, 1 - shrink)
+        x_integrate = numpy.linspace(xfrom, xto, 100_000)
+        area = scipy.integrate.cumtrapz(self.likelihood(x=x_integrate, y=y, scan_x=True), x_integrate, initial=0)
+        cdf = (area / area[-1]) * (1 - shrink) + shrink / 2
+
+        # TODO: create a smart x-vector from the CDF with varying stepsize
+
+        if ci_prob != 1:
+            if not isinstance(ci_prob, (int, float)) or not (0 < ci_prob <= 1):
+                raise ValueError(f'Unexpected `ci_prob` value of {ci_prob}. Expected float in interval (0, 1].')
+
+            # determine the interval bounds from the high-resolution CDF
+            eti_lower, eti_upper = _get_eti(x_integrate, cdf, ci_prob)
+            hdi_lower, hdi_upper = _get_hdi(x_integrate, cdf, ci_prob, eti_lower, eti_upper, history=None)
+
+            eti_x = numpy.linspace(eti_lower, eti_upper, steps)
+            hdi_x = numpy.linspace(hdi_lower, hdi_upper, steps)
+            eti_pdf = self.likelihood(x=eti_x, y=y, scan_x=True) / likelihood_integral
+            hdi_pdf = self.likelihood(x=hdi_x, y=y, scan_x=True) / likelihood_integral
+            eti_prob = _interval_prob(x_integrate, cdf, eti_lower, eti_upper)
+            hdi_prob = _interval_prob(x_integrate, cdf, hdi_lower, hdi_upper)
+        else:
+            x = numpy.linspace(lower, upper, steps)
+            eti_x = hdi_x = x
+            eti_pdf = hdi_pdf = self.likelihood(x=x, y=y, scan_x=True) / likelihood_integral
+            eti_prob = hdi_prob = 1
+
+        median = x_integrate[numpy.argmin(numpy.abs(cdf - 0.5))]
+
+        return ContinuousUnivariateInference(
+            median,
+            eti_x=eti_x, eti_pdf=eti_pdf, eti_prob=eti_prob,
+            hdi_x=hdi_x, hdi_pdf=hdi_pdf, hdi_prob=hdi_prob,
+        )
+
+
+class ContinuousMultivariateModel(CalibrationModel):
+    def __init__(self, independent_key: str, dependent_key: str, *, theta_names: typing.Tuple[str], ndim: int):
+        super().__init__(independent_key, dependent_key, theta_names=theta_names, ndim=ndim)
+
+    def infer_independent(self, y: Union[int, float, numpy.ndarray], *, lower, upper) -> ContinuousMultivariateInference:
+        return super().infer_independent(y, lower=lower, upper=upper)
+
+
 def logistic(x, theta):
     """4-parameter logistic model.
         
@@ -913,3 +1142,47 @@ def polynomial(x, theta):
     """
     # Numpy's polynomial function wants to get the highest degree first
     return numpy.polyval(theta[::-1], x)
+
+
+def exponential(x, theta):
+    """3-parameter exponential model.
+    
+    Parameters
+    ----------
+    x : array-like
+        independent variable
+    theta : array-like
+        Parameters of the exponential model:
+        - I: y-axis intercept
+        - L: asymptotic limit
+        - k: kinetic rate
+    
+    Returns
+    -------
+    y : array-like
+        dependent variable
+    """
+    I,L,k = theta[:3]
+    return (L-I) *(1-numpy.exp(-k*x)) + I
+
+
+def inverse_exponential(y, theta):
+    """Inverse of 3-parameter exponential model.
+    
+    Parameters
+    ----------
+    y : array-like
+        dependent variable
+    theta : array-like
+        Parameters of the exponential model:
+        - I: y-axis intercept
+        - L: asymptotic limit
+        - k: kinetic rate
+    
+    Returns
+    -------
+    x : array-like
+        independent variable
+    """
+    I,L,k = theta[:3]
+    return -1/k * numpy.log(1-(y-I)/(L-I))
